

/***
 * Soundcard receiver code
 * Copyright (C) 2014-2016 Knut R Leer (krleer@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Based on the sr4.c recorder example.
 */


#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include <alsa/asoundlib.h>

int G_looping;

void sigintcatcher(int sig, siginfo_t *siginfo, void *vpp)
{
	fprintf(stderr, "%s(%d): pid=%u: Caught SIGINT %d\n",
		__FILE__, __LINE__, getpid(), sig);
	G_looping = 0;

	// _exit(sig);
}

int main(int argc, char **argv)
{
	struct sigaction sact;
	int sas;	
	int srate, j;

	// long loops;
	int rc;
	int size;
	snd_pcm_t *handle;
	snd_pcm_hw_params_t *params;
	unsigned int val;
	int dir;
	snd_pcm_uframes_t frames;
	char *buffer;

	char devicename[80];

	G_looping = 1;

    memset(&sact, 0, sizeof(struct sigaction));
    sact.sa_sigaction = sigintcatcher;
    sact.sa_flags = SA_SIGINFO | SA_RESTART;
    sas = sigaction(SIGINT, &sact, 0);
    fprintf(stderr, "%s(%d): pid=%u: Set siginthandler = %d\n", 
		__FILE__, __LINE__, getpid(), sas); 
    if(sas == -1) exit(1);

	srate = 48000;
	snprintf(devicename, 80, "hw:1,0");

	for(j = 1; j<argc; j++)
	{
		if(!strcmp("-r", argv[j]))
		{
			if(j+1 < argc)
			{
				srate = strtol(argv[j+1], NULL, 0); 
				j++;
			}
		}

		if(!strcmp("-D", argv[j]))
		{
			if(j+1 < argc)
			{
				snprintf(devicename, 80, "%s", argv[j+1]);
				j++;
			}
		}
	} 

	/* Open the device */

	/* Open PCM device for recording (capture). */
	rc = snd_pcm_open(&handle, devicename, SND_PCM_STREAM_CAPTURE, 0);

	if (rc < 0) 
	{
		fprintf(stderr, "%s(%d): unable to open pcm device %s : %s\n",
			__FILE__, __LINE__, devicename, snd_strerror(rc));
		exit(1);
	}

	/* Allocate a hardware parameters object. */
	snd_pcm_hw_params_alloca(&params);

	/* Fill it in with default values. */
	snd_pcm_hw_params_any(handle, params);

	/* Set the desired hardware parameters. */

	/* Interleaved mode */
	snd_pcm_hw_params_set_access(handle, params, SND_PCM_ACCESS_RW_INTERLEAVED);

	/* Signed 16-bit little-endian format */
	snd_pcm_hw_params_set_format(handle, params, SND_PCM_FORMAT_S16_LE);

	/* Two channels (stereo) */
	snd_pcm_hw_params_set_channels(handle, params, 2);

	/* 44100 bits/second sampling rate (CD quality) */
	/* 48000 bits/second sampling rate (CD quality) */
	val = srate;
	snd_pcm_hw_params_set_rate_near(handle, params, &val, &dir);

	fprintf(stderr,
		"%s(%d): srate set %d actual %d\n",
		__FILE__, __LINE__, srate, val);

	/* Set period size to 32 frames. */
	frames = 32;
	snd_pcm_hw_params_set_period_size_near(handle, params, &frames, &dir);

	/* Write the parameters to the driver */
	rc = snd_pcm_hw_params(handle, params);
	if (rc < 0) 
	{
		fprintf(stderr,
			"%s(%d): unable to set hw parameters: %s\n",
			__FILE__, __LINE__, snd_strerror(rc));
		exit(1);
	}

	/* Use a buffer large enough to hold one period */
	snd_pcm_hw_params_get_period_size(params, &frames, &dir);

	size = frames * 4; /* 2 bytes/sample, 2 channels */
	buffer = (char *) malloc(size);

	fprintf(stderr, "%s(%d): Alloc %d \n", __FILE__, __LINE__, size);

	/* We want to loop for 5 seconds ...  */
	snd_pcm_hw_params_get_period_time(params, &val, &dir);

	// loops = 5000000 / val;
	// while(loops > 0) 

	while(G_looping)
	{
		// loops--;
		rc = snd_pcm_readi(handle, buffer, frames);
		if(rc == -EPIPE) 
		{
			/* EPIPE means overrun */
			fprintf(stderr, "%s(%d): overrun occurred\n", __FILE__, __LINE__);
			snd_pcm_prepare(handle);
		} 
		else 
		if(rc < 0) 
		{
			fprintf(stderr,
			"%s(%d): error from read: %s\n",
			__FILE__, __LINE__, snd_strerror(rc));
		} 
		else 
		if(rc != (int)frames) 
		{
			fprintf(stderr, "%s(%d): short read, read %d frames\n", 
			__FILE__, __LINE__, 
			rc);
		}
		rc = write(1, buffer, size);
		if(rc != size)
		{
			fprintf(stderr,
			"%s(%d): short write: wrote %d bytes\n", 
			__FILE__, __LINE__, rc);
		}
	}
	snd_pcm_drain(handle);
	snd_pcm_close(handle);
	free(buffer);

	return 0;
}



