/***
 * SoftRock UDP client control program
 * Copyright (C) 2016 Knut R Leer (krleer@gmail.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Works against the usbsoftrock server code written by Andrew Nilsson
 * Based on powerSwitch.c by Christian Starkjohann,
 * and usbtemp.c by Mathias Dalheimer
 * of Objective Development Software GmbH (2005)
 * (see http://www.obdev.at/avrusb)
 */


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <errno.h>

#include "udpconv.h"

int main(int argc, char **argv)
{
	int ee;
	uint16_t portno;
	char ipadtxt[40];
	char cmd[100];
	int rxc;
	char repl[100];
	char ftset[30];
	int j;
	int sfts;
	char *sp, *stop;
	double fam;
	int verbose;
	int shortout;

	sfts = 0;
	verbose = 0;
	shortout = 0;

	portno = 19004;
	snprintf(ipadtxt, 40, "%s", "127.0.0.1"); 

	for(j = 1; j<argc; j++)
	{
		if(!strcmp("-?", argv[j]))
		{
			printf("Arguments:\n"
				"-h host - (default %s)\n"
				"-p port - (default %d)\n"
				"-s      - brief output in Hz\n"
				"-v      - verbose\n"
				"-fm frequency - set frequency in MHz\n"
				"-fk frequency - set frequency in kHz\n"
				"-fh frequency - set frequency in Hz\n"
			, ipadtxt, portno);  

		}

		if(!strcmp("-v", argv[j]))
		{
			verbose = 1;
		}


		if(!strcmp("-s", argv[j]))
		{
			shortout = 1;
		}


		if(!strcmp("-p", argv[j]))
		{
			if(j+1<argc)
				portno = strtol(argv[j+1], &stop, 10); 
		}

		if(!strcmp("-h", argv[j]))
		{
			if(j+1<argc)
				snprintf(ipadtxt, 40, "%s", argv[j+1]);
		}

		if(!strcmp("-fm", argv[j]))
		{
			if(j+1<argc)
				snprintf(ftset, 30, "%s", argv[j+1]);
			sfts = 1;
		}

		if(!strcmp("-fh", argv[j]))
		{
			if(j+1<argc)
			{
				fam = strtod(argv[j+1], &stop);

				snprintf(ftset, 30, "%f", fam/1000000.0);
			}
			sfts = 1;
		}

		if(!strcmp("-fk", argv[j]))
		{
			if(j+1<argc)
			{
				fam = strtod(argv[j+1], &stop);

				snprintf(ftset, 30, "%f", fam/1000.0);
			}
			sfts = 1;
		}
	}

	if(sfts)
	{
		snprintf(cmd, 100, "set freq %s", ftset);
		rxc = udpconv(ipadtxt, portno, cmd, repl, 100, &ee);
		if(verbose)
		{
			if(ee != 0)
				printf("Received count = %d E = %d (%s)\n", rxc, ee, strerror(ee)); 
			printf("sent %s received %s (%d)\n", cmd, repl, rxc); 
		}
	}
	else
	{
		snprintf(cmd, 100, "get freq");
		rxc = udpconv(ipadtxt, portno, cmd, repl, 100, &ee);
		if(verbose)
		{
			if(ee != 0)
				printf("Received count = %d E = %d (%s)\n", rxc, ee, strerror(ee)); 
			printf("sent %s received <%s> (%d)\n", cmd, repl, rxc); 
		}
		sp = strchr(repl, ' ');
		if(sp)
		{
			fam = strtod(sp, &stop);
			if(shortout)
				printf("%.0f\n", fam * 1000000.0); 
			else
				printf("Now at %G MHz\n", fam); 
		}
	}
	return(0);
}



